use std::{backtrace::Backtrace, io};

use thiserror::Error;

#[derive(Debug, Error)]
pub enum Error {
    #[error("IO error: {source}")]
    Io {
        #[from]
        source: io::Error,
        backtrace: Backtrace,
    },

    #[error("Script error: {message}")]
    ScriptError {
        message: String,
        backtrace: Backtrace,
    },

    #[error("Run error: {message}")]
    ExecutionError {
        message: String,
        backtrace: Backtrace,
    },

    #[error("Deploy error: {message}")]
    DeployError {
        message: String,
        backtrace: Backtrace,
    },

    #[error("Format error: {message}")]
    FormatError {
        message: String,
        backtrace: Backtrace,
    },

    #[error("JSON error: {source}")]
    JSONError {
        #[from]
        source: serde_json::Error,
        backtrace: Backtrace,
    },
}

pub type Result<T> = std::result::Result<T, Error>;
